data {
	int<lower=0> N;
	real<lower=1.0> x[N];
	real<lower=0> minx;
}
parameters { 
	real<lower=0.0001,upper=minx>   alpha;
	real<lower=0.0001>  beta; 
}
model {
       for(n in 1:N){
	   x[n] ~ pareto(alpha, beta);
	 }
}
generated quantities{
	real E;
	real med;
	real sa;
	real u;
	real<lower=0> q90;
 	vector[N] log_lik;
	
	for(n in 1:N){
		log_lik[n] = pareto_lpdf(x[n]|alpha,beta);
	}
	E = (beta*alpha)/(beta-1);
	med=2^(1/beta)*alpha;
	sa=E-med;
	u=step(sa-1.0);
	q90 = alpha/((1-0.90)^(1/beta));
}
