#stan version 2.10
library(rstan)
library(pomp)

################################################################################
###single group
###Data
source("data16.R")
dat_ho <-list(I=length(height_outf), y=height_outf)
dat_wo <-list(I=length(weight_outf), y=weight_outf)
dat_hi <-list(I=length(height_inf), y=height_inf)
dat_wi <-list(I=length(weight_inf), y=weight_inf)

###setting
war<-5000
ite<-10000
see<-1234
dig<-3
cha<-4

###Normal Distribution
###stan
scrM01 <- "code16_1.stan"
parM01 <- c("mu1","mu2","log_lik")
rtM01 <- stanc(file = scrM01, model_name = scrM01, verbose=T)
smM01 <- stan_model(stanc_ret = rtM01, verbose = FALSE)

fit_hoM01 <- sampling(smM01,data=dat_ho,iter=ite,chains=cha,warmup=war, seed=see,pars=parM01)
fit_woM01 <- sampling(smM01,data=dat_wo,iter=ite,chains=cha,warmup=war, seed=see,pars=parM01)
fit_hiM01 <- sampling(smM01,data=dat_hi,iter=ite,chains=cha,warmup=war, seed=see,pars=parM01)
fit_wiM01 <- sampling(smM01,data=dat_wi,iter=ite,chains=cha,warmup=war, seed=see,pars=parM01)

###result
res_hoM01 <- summary(fit_hoM01)$summary
res_woM01 <- summary(fit_woM01)$summary
res_hiM01 <- summary(fit_hiM01)$summary
res_wiM01 <- summary(fit_wiM01)$summary

round(res_hoM01,2);round(res_woM01,2)
round(res_hiM01,2);round(res_wiM01,2)

ext_hoM01 <- extract(fit_hoM01, parM01);ll_hoM01<-ext_hoM01$log_lik
ext_woM01 <- extract(fit_woM01, parM01);ll_woM01<-ext_woM01$log_lik
ext_hiM01 <- extract(fit_hiM01, parM01);ll_hiM01<-ext_hiM01$log_lik
ext_wiM01 <- extract(fit_wiM01, parM01);ll_wiM01<-ext_wiM01$log_lik

##WAIC
lppd_hoM01 <- logmeanexp(ll_hoM01);p_waic_hoM01 <- var(ll_hoM01)
lppd_woM01 <- logmeanexp(ll_woM01);p_waic_woM01 <- var(ll_woM01)
lppd_hiM01 <- logmeanexp(ll_hiM01);p_waic_hiM01 <- var(ll_hiM01)
lppd_wiM01 <- logmeanexp(ll_wiM01);p_waic_wiM01 <- var(ll_wiM01)

(waic_hoM01 <- (-2)*lppd_hoM01 + 2*p_waic_hoM01)
(waic_woM01 <- (-2)*lppd_woM01 + 2*p_waic_woM01)
(waic_hiM01 <- (-2)*lppd_hiM01 + 2*p_waic_hiM01)
(waic_wiM01 <- (-2)*lppd_wiM01 + 2*p_waic_wiM01)

closeAllConnections()

###Skew Normal Distribution
###stan
scrS01<-"code16_2.stan"
parS01<-c("mu1","mu2","mu3","log_lik")
rtS01 <- stanc(file = scrS01, model_name = scrS01, verbose=T)
smS01 <- stan_model(stanc_ret = rtS01, verbose = FALSE)

fit_hoS01 <- sampling(smS01,data=dat_ho,iter=ite,chains=cha,warmup=war, seed=see,pars=parS01)
fit_woS01 <- sampling(smS01,data=dat_wo,iter=ite,chains=cha,warmup=war, seed=see,pars=parS01)
fit_hiS01 <- sampling(smS01,data=dat_hi,iter=ite,chains=cha,warmup=war, seed=see,pars=parS01)
fit_wiS01 <- sampling(smS01,data=dat_wi,iter=ite,chains=cha,warmup=war, seed=see,pars=parS01)

###result
res_hoS01 <- summary(fit_hoS01)$summary
res_woS01 <- summary(fit_woS01)$summary
res_hiS01 <- summary(fit_hiS01)$summary
res_wiS01 <- summary(fit_wiS01)$summary

round(res_hoS01,2);round(res_woS01,2)
round(res_hiS01,2);round(res_wiS01,2)

ext_hoS01 <- extract(fit_hoS01, parS01);ll_hoS01<-ext_hoS01$log_lik
ext_woS01 <- extract(fit_woS01, parS01);ll_woS01<-ext_woS01$log_lik
ext_hiS01 <- extract(fit_hiS01, parS01);ll_hiS01<-ext_hiS01$log_lik
ext_wiS01 <- extract(fit_wiS01, parS01);ll_wiS01<-ext_wiS01$log_lik

##WAIC
lppd_hoS01 <- logmeanexp(ll_hoS01);p_waic_hoS01 <- var(ll_hoS01)
lppd_woS01 <- logmeanexp(ll_woS01);p_waic_woS01 <- var(ll_woS01)
lppd_hiS01 <- logmeanexp(ll_hiS01);p_waic_hiS01 <- var(ll_hiS01)
lppd_wiS01 <- logmeanexp(ll_wiS01);p_waic_wiS01 <- var(ll_wiS01)

(waic_hoS01 <- (-2)*lppd_hoS01 + 2*p_waic_hoS01)
(waic_woS01 <- (-2)*lppd_woS01 + 2*p_waic_woS01)
(waic_hiS01 <- (-2)*lppd_hiS01 + 2*p_waic_hiS01)
(waic_wiS01 <- (-2)*lppd_wiS01 + 2*p_waic_wiS01)

WAIC <- matrix(c(
waic_hoM01,waic_hoS01,waic_woM01,waic_woS01,
waic_hiM01,waic_hiS01,waic_wiM01,waic_wiS01),2,4,byrow=T)



################################################################################
###multi group
###Data
dat_woi <-list(I1=length(weight_outf), y1=weight_outf,
			   I2=length(weight_inf), y2=weight_inf)

###setting
war<-5000
ite<-10000
see<-1234
dig<-3
cha<-4

###Normal Distribution
###stan
parSM<-c("mu1_g1","mu2_g1","mu3_g1","mu1_g2","mu2_g2","mu3_g2","log_lik")

##Equal non
scrSM000<-"code16_3.stan"
rtSM000 <- stanc(file = scrSM000, model_name = scrSM000, verbose=T)
smSM000 <- stan_model(stanc_ret = rtSM000, verbose = FALSE)
fit_woiSM000 <- sampling(smSM000,data=dat_woi,iter=ite,chains=cha,warmup=war, seed=see,pars=parSM)

##Equal skew
scrSM001<-"code16_4.stan"
rtSM001 <- stanc(file = scrSM001, model_name = scrSM001, verbose=T)
smSM001 <- stan_model(stanc_ret = rtSM001, verbose = FALSE)
fit_woiSM001 <- sampling(smSM001,data=dat_woi,iter=ite,chains=cha,warmup=war, seed=see,pars=parSM)

##Equal var
scrSM010<-"code16_5.stan"
rtSM010 <- stanc(file = scrSM010, model_name = scrSM010, verbose=T)
smSM010 <- stan_model(stanc_ret = rtSM010, verbose = FALSE)
fit_woiSM010 <- sampling(smSM010,data=dat_woi,iter=ite,chains=cha,warmup=war, seed=see,pars=parSM)

##Equal var skew
scrSM011<-"code16_6.stan"
rtSM011 <- stanc(file = scrSM011, model_name = scrSM011, verbose=T)
smSM011 <- stan_model(stanc_ret = rtSM011, verbose = FALSE)
fit_woiSM011 <- sampling(smSM011,data=dat_woi,iter=ite,chains=cha,warmup=war, seed=see,pars=parSM)

closeAllConnections()
##Equal mean
scrSM100<-"code16_7.stan"
rtSM100 <- stanc(file = scrSM100, model_name = scrSM100, verbose=T)
smSM100 <- stan_model(stanc_ret = rtSM100, verbose = FALSE)
fit_woiSM100 <- sampling(smSM100,data=dat_woi,iter=ite,chains=cha,warmup=war, seed=see,pars=parSM)

##Equal mean skew
scrSM101<-"code16_8.stan"
rtSM101 <- stanc(file = scrSM101, model_name = scrSM101, verbose=T)
smSM101 <- stan_model(stanc_ret = rtSM101, verbose = FALSE)
fit_woiSM101 <- sampling(smSM101,data=dat_woi,iter=ite,chains=cha,warmup=war, seed=see,pars=parSM)

##Equal mean var
scrSM110<-"code16_9.stan"
rtSM110 <- stanc(file = scrSM110, model_name = scrSM110, verbose=T)
smSM110 <- stan_model(stanc_ret = rtSM110, verbose = FALSE)
fit_woiSM110 <- sampling(smSM110,data=dat_woi,iter=ite,chains=cha,warmup=war, seed=see,pars=parSM)

##Equal mean var skew
scrSM111<-"code16_10.stan"
rtSM111 <- stanc(file = scrSM111, model_name = scrSM111, verbose=T)
smSM111 <- stan_model(stanc_ret = rtSM111, verbose = FALSE)
fit_woiSM111 <- sampling(smSM111,data=dat_woi,iter=ite,chains=cha,warmup=war, seed=see,pars=parSM)

###result
res_woiSM000 <- summary(fit_woiSM000)$summary
res_woiSM001 <- summary(fit_woiSM001)$summary
res_woiSM010 <- summary(fit_woiSM010)$summary
res_woiSM011 <- summary(fit_woiSM011)$summary
res_woiSM100 <- summary(fit_woiSM100)$summary
res_woiSM101 <- summary(fit_woiSM101)$summary
res_woiSM110 <- summary(fit_woiSM110)$summary
res_woiSM111 <- summary(fit_woiSM111)$summary

round(res_woiSM000,2);round(res_woiSM001,2);
round(res_woiSM010,2);round(res_woiSM011,2)
round(res_woiSM100,2);round(res_woiSM101,2);
round(res_woiSM110,2);round(res_woiSM111,2)

ext_woiSM000 <- extract(fit_woiSM000, parSM);ll_woiSM000 <- ext_woiSM000$log_lik
ext_woiSM001 <- extract(fit_woiSM001, parSM);ll_woiSM001 <- ext_woiSM001$log_lik
ext_woiSM010 <- extract(fit_woiSM010, parSM);ll_woiSM010 <- ext_woiSM010$log_lik
ext_woiSM011 <- extract(fit_woiSM011, parSM);ll_woiSM011 <- ext_woiSM011$log_lik
ext_woiSM100 <- extract(fit_woiSM100, parSM);ll_woiSM100 <- ext_woiSM100$log_lik
ext_woiSM101 <- extract(fit_woiSM101, parSM);ll_woiSM101 <- ext_woiSM101$log_lik
ext_woiSM110 <- extract(fit_woiSM110, parSM);ll_woiSM110 <- ext_woiSM110$log_lik
ext_woiSM111 <- extract(fit_woiSM111, parSM);ll_woiSM111 <- ext_woiSM111$log_lik

##WAIC
lppd_woiSM000 <- logmeanexp(ll_woiSM000);p_waic_woiSM000 <- var(ll_woiSM000)
lppd_woiSM001 <- logmeanexp(ll_woiSM001);p_waic_woiSM001 <- var(ll_woiSM001)
lppd_woiSM010 <- logmeanexp(ll_woiSM010);p_waic_woiSM010 <- var(ll_woiSM010)
lppd_woiSM011 <- logmeanexp(ll_woiSM011);p_waic_woiSM011 <- var(ll_woiSM011)
lppd_woiSM100 <- logmeanexp(ll_woiSM100);p_waic_woiSM100 <- var(ll_woiSM100)
lppd_woiSM101 <- logmeanexp(ll_woiSM101);p_waic_woiSM101 <- var(ll_woiSM101)
lppd_woiSM110 <- logmeanexp(ll_woiSM110);p_waic_woiSM110 <- var(ll_woiSM110)
lppd_woiSM111 <- logmeanexp(ll_woiSM111);p_waic_woiSM111 <- var(ll_woiSM111)

(waic_woiSM000<- (-2)*lppd_woiSM000 + 2*p_waic_woiSM000)
(waic_woiSM001<- (-2)*lppd_woiSM001 + 2*p_waic_woiSM001)
(waic_woiSM010<- (-2)*lppd_woiSM010 + 2*p_waic_woiSM010)
(waic_woiSM011<- (-2)*lppd_woiSM011 + 2*p_waic_woiSM011)
(waic_woiSM100<- (-2)*lppd_woiSM100 + 2*p_waic_woiSM100)
(waic_woiSM101<- (-2)*lppd_woiSM101 + 2*p_waic_woiSM101)
(waic_woiSM110<- (-2)*lppd_woiSM110 + 2*p_waic_woiSM110)
(waic_woiSM111<- (-2)*lppd_woiSM111 + 2*p_waic_woiSM111)

WAIC_woi <- matrix(c(
waic_woiSM000,waic_woiSM001,waic_woiSM010,waic_woiSM011,
waic_woiSM100,waic_woiSM101,waic_woiSM110,waic_woiSM111),2,4,byrow=T)
