data {
  int<lower=1> K;  
  int<lower=1> V;  
  int<lower=0> T;  
  int<lower=1> T_unsup;  
  int<lower=1,upper=V> x[T]; 
  int<lower=1,upper=K> z[T]; 
  int<lower=1,upper=V> x_u[T_unsup]; 
  vector<lower=0>[K] prior_a;  
  vector<lower=0>[V] prior_b;  
}
parameters {
  simplex[K] a[K];  
  simplex[V] b[K];    
}
model {
  for (i in 1:K) 
    a[i] ~ dirichlet(prior_a);
  for (j in 1:K)
    b[j] ~ dirichlet(prior_b);
  for (t in 1:T)
    x[t] ~ categorical(b[z[t]]);
  for (t in 2:T)
    z[t] ~ categorical(a[z[t-1]]);

  { 
    real acc[K];
    real alpha[T_unsup,K];
    for (i in 1:K)
      alpha[1,i] = log(b[i,x_u[1]]);
    for (t in 2:T_unsup) {
      for (j in 1:K) {
        for (i in 1:K)
          acc[i] = alpha[t-1,i] + log(a[i,j]) + log(b[j,x_u[t]]);
          alpha[t,j] = log_sum_exp(acc);
      }
    }
    target += log_sum_exp(alpha[T_unsup]);
  }
}
generated quantities {
  int<lower=1,upper=K> z_hat[T_unsup];
  real zeta;
  real best_logp[T_unsup,K];
  int  psi[T_unsup,K];
  real u1;
  real u2;
  real u3;
   

    for (i in 1:K)
      best_logp[1,K] = log(b[i,x_u[1]]);
    for (t in 2:T_unsup) {
      for (j in 1:K) {
        best_logp[t,j] = negative_infinity();
        for (i in 1:K) {
          real logp;
          logp = best_logp[t-1,i] + log(a[i,j]) + log(b[j,x_u[t]]);
          if (logp > best_logp[t,j]) {
            psi[t,j] = i;
            best_logp[t,j] = logp;
          }
        }
      }
    }
    zeta = max(best_logp[T_unsup]);
    for (j in 1:K){
      if (best_logp[T_unsup,j] == zeta)
        z_hat[T_unsup] = j;
    } 
    for (t in 1:(T_unsup - 1)){
      z_hat[T_unsup - t] = psi[T_unsup - t + 1, 
                                      z_hat[T_unsup - t + 1]];
    }
     u1 = z_hat[10]==1 ? 1 : 0;
     u2 = z_hat[10]==2 ? 1 : 0;
     u3 = z_hat[10]==3 ? 1 : 0;

}

