#stan version 2.10
library(rstan)

###Data
source("data34.R")
ntrials <- 30
maxpump <- 16
p <- rep(.2,ntrials)
data01 <- list(ntrials=ntrials, p=p, maxpump=maxpump, options=Options[,1], d=Dis[[1]]) 

###setting
war<-2500
ite<-5000
see<-1234
dig<-3
cha<-4

###stan
scr01 <- "code34_1.stan"
par01 <- c("gamma", "beta")
#myinits <- list(list(gplus=1.2, beta=.5))
rt01 <- stanc(file = scr01, model_name = scr01, verbose=T)
sm01 <- stan_model(stanc_ret = rt01, verbose = FALSE)
fit01 <- sampling(sm01,data=data01,iter=ite,chains=cha,warmup=war, seed=see,pars=par01)

###result
res01 <- summary(fit01)$summary
round(res01,3)
gamma_estS <- res01[grep("gamma", rownames(res01)),1]
beta_estS <- res01[grep("beta", rownames(res01)),1]


###plot
##function of theta
theta_jp <- function(k,beta,gamma,p){
	omega <- -gamma/(log(1-p))
	theta <- 1/(1+exp(beta*(k-omega)))
	return(theta)
}

k <- 1:25
par(mfrow=c(1,2))
hist(Npumps[,1],breaks=c(5:16),xlab="pv",main="")
plot(k,theta_jp(k,beta=beta_estS,gamma=gamma_estS,p=0.15),type="b",pch="1",ylim=c(0,1),ylab="theta",las=1,lwd=2.5,
xlab="pv")
par(mfrow=c(1,1))

###MultiModel
###Data
data02 <- list(ntrials=ntrials, p=p, maxpump=maxpump, N=7, 
options=t(Options[,c(1,2,3,4,5,6,7)]), d=structure(.Data=c(Dis[[1]],Dis[[2]],Dis[[3]],Dis[[4]],Dis[[5]],Dis[[6]],Dis[[7]]),.Dim=c(30,16,7)))

###setting
war<-2500
ite<-5000
see<-1234
dig<-3
cha<-4

###stan
scr02 <- "code34_2.stan"
par02 <- c("gamma", "beta","mu_g","sigma_g","mu_b","sigma_b")
myinits02 <- rep(list(list(gamma=rep(1.5,7), beta=rep(1.0,7))),cha)
rt02 <- stanc(file = scr02, model_name = scr02, verbose=T)
sm02 <- stan_model(stanc_ret = rt02, verbose = FALSE)
fit02 <- sampling(sm02,data=data02,iter=ite,chains=cha,warmup=war, seed=see,pars=par02,init=myinits02)

###result
res02 <- summary(fit02)$summary
round(res02,3)

gamma_est <- res02[grep("gamma", rownames(res02)),1]
beta_est <- res02[grep("beta", rownames(res02)),1]

k <- 1:25
plot(k,theta_jp(k,beta=beta_est[1],gamma=gamma_est[1],p=0.15),type="b",pch="1",ylim=c(0,1),ylab="theta",las=1,lwd=2.5);par(new=T)
plot(k,theta_jp(k,beta=beta_est[2],gamma=gamma_est[2],p=0.15),type="b",pch="2",ylim=c(0,1),ylab="theta",las=1,lwd=2.5);par(new=T)
plot(k,theta_jp(k,beta=beta_est[3],gamma=gamma_est[3],p=0.15),type="b",pch="3",ylim=c(0,1),ylab="theta",las=1,lwd=2.5);par(new=T)
plot(k,theta_jp(k,beta=beta_est[4],gamma=gamma_est[4],p=0.15),type="b",pch="4",ylim=c(0,1),ylab="theta",las=1,lwd=2.5);par(new=T)
plot(k,theta_jp(k,beta=beta_est[5],gamma=gamma_est[5],p=0.15),type="b",pch="5",ylim=c(0,1),ylab="theta",las=1,lwd=2.5);par(new=T)
plot(k,theta_jp(k,beta=beta_est[6],gamma=gamma_est[6],p=0.15),type="b",pch="6",ylim=c(0,1),ylab="theta",las=1,lwd=2.5);par(new=T)
plot(k,theta_jp(k,beta=beta_est[7],gamma=gamma_est[7],p=0.15),type="b",pch="7",ylim=c(0,1),ylab="theta",las=1,lwd=2.5)
